@@ -95,6 +95,16 @@ class VMAXISCSIDriver(san.SanISCSIDriver):
               - Support for manage/unmanage snapshots
                 (vmax-manage-unmanage-snapshot)
               - Support for revert to volume snapshot
+        backport from 3.2.0
+              - Fix for SSL verification/cert application (bug #1772924)
+              - Incorrect condition for an empty list (bug #1787219)
+              - Deleting snapshot that is source of multiple volumes fails
+                (bug #1768047)
+              - Block revert to snapshot for replicated volumes (bug #1777871)
+              - Fix for get-pools command (bug #1784856)
+        backport from 3.3.0
+              - Fix for initiator retrieval and short hostname unmapping
+                (bugs #1783855 #1783867)
     """
 
     VERSION = "3.1.0"
