@@ -34,6 +34,7 @@ requests.packages.urllib3.disable_warnings(urllib_exp.InsecureRequestWarning)
 LOG = logging.getLogger(__name__)
 SLOPROVISIONING = 'sloprovisioning'
 REPLICATION = 'replication'
+SYSTEM = 'system'
 U4V_VERSION = '84'
 UCODE_5978 = '5978'
 retry_exc_tuple = (exception.VolumeBackendAPIException,)
@@ -75,14 +76,10 @@ class VMAXRest(object):
         port = array_info['RestServerPort']
         self.user = array_info['RestUserName']
         self.passwd = array_info['RestPassword']
-        self.cert = array_info['SSLCert']
-        verify = array_info['SSLVerify']
-        if verify and verify.lower() == 'false':
-            verify = False
-        self.verify = verify
+        self.verify = array_info['SSLVerify']
         ip_port = "%(ip)s:%(port)s" % {'ip': ip, 'port': port}
-        self.base_uri = ("https://%(ip_port)s/univmax/restapi"
-                         % {'ip_port': ip_port})
+        self.base_uri = ("https://%(ip_port)s/univmax/restapi" % {
+            'ip_port': ip_port})
         self.session = self._establish_rest_session()
 
     def _establish_rest_session(self):
@@ -97,8 +94,6 @@ class VMAXRest(object):
         session.auth = requests.auth.HTTPBasicAuth(self.user, self.passwd)
         if self.verify is not None:
             session.verify = self.verify
-        if self.cert:
-            session.cert = self.cert
 
         return session
 
@@ -464,9 +459,16 @@ class VMAXRest(object):
         :returns: slo_list -- list of service level names
         """
         slo_list = []
-        slo_dict = self.get_resource(array, SLOPROVISIONING, 'slo')
+        slo_dict = self.get_resource(array, SLOPROVISIONING, 'slo',
+                                     version='90')
         if slo_dict and slo_dict.get('sloId'):
-            slo_list = slo_dict['sloId']
+            if any(self.get_vmax_model(array) in x for x in
+                   utils.VMAX_AFA_MODELS):
+                if 'Optimized' in slo_dict.get('sloId'):
+                    slo_dict['sloId'].remove('Optimized')
+            for slo in slo_dict['sloId']:
+                if slo and slo not in slo_list:
+                    slo_list.append(slo)
         return slo_list
 
     def get_workload_settings(self, array):
@@ -484,6 +486,20 @@ class VMAXRest(object):
                 workload_setting = wl_details['workloadId']
         return workload_setting
 
+    def get_vmax_model(self, array):
+        """Get the VMAX model.
+
+        :param array: the array serial number
+        :return: the VMAX model
+        """
+        vmax_version = ''
+        system_uri = ("/%(version)s/system/symmetrix/%(array)s" % {
+            'version': U4V_VERSION, 'array': array})
+        system_info = self._get_request(system_uri, SYSTEM)
+        if system_info and system_info.get('model'):
+            vmax_version = system_info.get('model')
+        return vmax_version
+
     def is_compression_capable(self, array):
         """Check if array is compression capable.
 
@@ -717,12 +733,14 @@ class VMAXRest(object):
         volume_dict = {'array': array, 'device_id': device_id}
         return volume_dict
 
-    def check_volume_device_id(self, array, device_id, volume_id):
+    def check_volume_device_id(self, array, device_id, volume_id,
+                               name_id=None):
         """Check if the identifiers match for a given volume.
 
         :param array: the array serial number
         :param device_id: the device id
         :param volume_id: cinder volume id
+        :param name_id: name id - used in host_assisted migration, optional
         :returns: found_device_id
         """
         element_name = self.utils.get_volume_element_name(volume_id)
@@ -736,6 +754,11 @@ class VMAXRest(object):
                        'di': device_id, 'vd': vol_details})
             if vol_identifier == element_name:
                 found_device_id = device_id
+            elif name_id:
+                # This may be host-assisted migration case
+                element_name = self.utils.get_volume_element_name(name_id)
+                if vol_identifier == element_name:
+                    found_device_id = device_id
         return found_device_id
 
     def add_vol_to_sg(self, array, storagegroup_name, device_id, extra_specs):
@@ -1301,17 +1324,6 @@ class VMAXRest(object):
             init_list = []
         return init_list
 
-    def get_in_use_initiator_list_from_array(self, array):
-        """Get the list of initiators which are in-use from the array.
-
-        Gets the list of initiators from the array which are in
-        hosts/ initiator groups.
-        :param array: the array serial number
-        :returns: init_list
-        """
-        params = {'in_a_host': 'true'}
-        return self.get_initiator_list(array, params)
-
     def get_initiator_group_from_initiator(self, array, initiator):
         """Given an initiator, get its corresponding initiator group, if any.
 
